@@ -30,6 +30,10 @@
 #include <math.h>
 #include <stdlib.h>
 
+#if MPT_COMPILER_MSVC
+#include <intrin.h>
+#endif
+
 
 
 OPENMPT_NAMESPACE_BEGIN
@@ -165,11 +169,11 @@ inline Tdst saturate_cast(Tsrc src)
 template <typename Tdst>
 inline Tdst saturate_cast(double src)
 {
-	if(src >= std::numeric_limits<Tdst>::max())
+	if(src >= static_cast<double>(std::numeric_limits<Tdst>::max()))
 	{
 		return std::numeric_limits<Tdst>::max();
 	}
-	if(src <= std::numeric_limits<Tdst>::min())
+	if(src <= static_cast<double>(std::numeric_limits<Tdst>::min()))
 	{
 		return std::numeric_limits<Tdst>::min();
 	}
@@ -179,11 +183,11 @@ inline Tdst saturate_cast(double src)
 template <typename Tdst>
 inline Tdst saturate_cast(float src)
 {
-	if(src >= std::numeric_limits<Tdst>::max())
+	if(src >= static_cast<float>(std::numeric_limits<Tdst>::max()))
 	{
 		return std::numeric_limits<Tdst>::max();
 	}
-	if(src <= std::numeric_limits<Tdst>::min())
+	if(src <= static_cast<float>(std::numeric_limits<Tdst>::min()))
 	{
 		return std::numeric_limits<Tdst>::min();
 	}
