@@ -595,12 +595,12 @@ mpt::ustring GetFullCreditsString()
 		"libopenmpt (based on OpenMPT / ModPlug Tracker)\n"
 #endif
 		"\n"
-		"Copyright \xC2\xA9 2004-2019 Contributors\n"
+		"Copyright \xC2\xA9 2004-2021 Contributors\n"
 		"Copyright \xC2\xA9 1997-2003 Olivier Lapicque\n"
 		"\n"
 		"Contributors:\n"
-		"Johannes Schultz (2008-2019)\n"
-		"J\xC3\xB6rn Heusipp (2012-2019)\n"
+		"Johannes Schultz (2008-2021)\n"
+		"J\xC3\xB6rn Heusipp (2012-2021)\n"
 		"Ahti Lepp\xC3\xA4nen (2005-2011)\n"
 		"Robin Fernandes (2004-2007)\n"
 		"Sergiy Pylypenko (2007)\n"
@@ -736,7 +736,8 @@ mpt::ustring GetFullCreditsString()
 #endif
 #if defined(MPT_WITH_LAME)
 		"The LAME project for LAME\n"
-		"http://lame.sourceforge.net/\n"
+		"https://lame.sourceforge.io/\n"
+		"\n"
 #endif
 #if defined(MPT_WITH_NLOHMANNJSON)
 		"Niels Lohmann et al. for nlohmann-json\n"
@@ -785,7 +786,7 @@ mpt::ustring GetFullCreditsString()
 mpt::ustring GetLicenseString()
 {
 	return MPT_UTF8(
-		"Copyright (c) 2004-2019, OpenMPT contributors" "\n"
+		"Copyright (c) 2004-2021, OpenMPT contributors" "\n"
 		"Copyright (c) 1997-2003, Olivier Lapicque" "\n"
 		"All rights reserved." "\n"
 		"" "\n"
