@@ -8,7 +8,7 @@
  */
 
 static const char * const license =
-"Copyright (c) 2004-2019, OpenMPT contributors" "\n"
+"Copyright (c) 2004-2021, OpenMPT contributors" "\n"
 "Copyright (c) 1997-2003, Olivier Lapicque" "\n"
 "All rights reserved." "\n"
 "" "\n"
@@ -70,6 +70,7 @@ static const char * const license =
 #include <conio.h>
 #include <fcntl.h>
 #include <io.h>
+#include <stdio.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <windows.h>
@@ -441,7 +442,7 @@ static std::string seconds_to_string( double time ) {
 
 static void show_info( std::ostream & log, bool verbose ) {
 	log << "openmpt123" << " v" << OPENMPT123_VERSION_STRING << ", libopenmpt " << openmpt::string::get( "library_version" ) << " (" << "OpenMPT " << openmpt::string::get( "core_version" ) << ")" << std::endl;
-	log << "Copyright (c) 2013-2019 OpenMPT developers <https://lib.openmpt.org/>" << std::endl;
+	log << "Copyright (c) 2013-2021 OpenMPT developers <https://lib.openmpt.org/>" << std::endl;
 	if ( !verbose ) {
 		log << std::endl;
 		return;
@@ -530,7 +531,7 @@ static void show_info( std::ostream & log, bool verbose ) {
 static void show_man_version( textout & log ) {
 	log << "openmpt123" << " v" << OPENMPT123_VERSION_STRING << std::endl;
 	log << std::endl;
-	log << "Copyright (c) 2013-2019 OpenMPT developers <https://lib.openmpt.org/>" << std::endl;
+	log << "Copyright (c) 2013-2021 OpenMPT developers <https://lib.openmpt.org/>" << std::endl;
 }
 
 static void show_short_version( textout & log ) {
@@ -2418,9 +2419,15 @@ static int main( int argc, char * argv [] ) {
 
 		log.writeout();
 
-		std::random_device rd;
-		std::seed_seq seq{ rd(), static_cast<unsigned int>( std::time( NULL ) ) };
-		std::default_random_engine prng( seq );
+		std::default_random_engine prng;
+		try {
+			std::random_device rd;
+			std::seed_seq seq{ rd(), static_cast<unsigned int>( std::time( NULL ) ) };
+			prng = std::default_random_engine{ seq };
+		} catch ( const std::exception & ) {
+			std::seed_seq seq{ static_cast<unsigned int>( std::time( NULL ) ) };
+			prng = std::default_random_engine{ seq };
+		}
 		std::srand( std::uniform_int_distribution<unsigned int>()( prng ) );
 
 		switch ( flags.mode ) {
