@@ -466,18 +466,16 @@ static PATTERNINDEX ConvertDMFPattern(FileReader &file, DMFPatternSettings &sett
 				//const int tickspeed = (tempoRealBPMmode) ? MAX(1, (tempoData * beat * 4) / 60) : tempoData;
 				const int tickspeed = (settings.realBPMmode) ? std::max(1, settings.tempoBPM * settings.beat * 2) : ((settings.tempoTicks + 1) * 30);
 				// Try to find matching speed - try higher speeds first, so that effects like arpeggio and tremor work better.
-				for(speed = 255; speed > 2; speed--)
+				for(speed = 255; speed >= 1; speed--)
 				{
 					// Original unoptimized formula:
 					// tempo = 30 * tickspeed * speed / 48;
 					tempo = tickspeed * speed / 48;
 					if(tempo >= 32 && tempo <= 255)
-					{
 						break;
-					}
 				}
 				Limit(tempo, 32, 255);
-				settings.internalTicks = (uint8)speed;
+				settings.internalTicks = static_cast<uint8>(std::max(1, speed));
 			} else
 			{
 				tempoChange = false;
@@ -642,10 +640,19 @@ static PATTERNINDEX ConvertDMFPattern(FileReader &file, DMFPatternSettings &sett
 
 					switch(effect2)
 					{
-					case 1:		// Note Finetune
-						effect2 = static_cast<ModCommand::COMMAND>(effectParam2 < 128 ? CMD_PORTAMENTOUP : CMD_PORTAMENTODOWN);
-						if(effectParam2 > 128) effectParam2 = 255 - effectParam2 + 1;
-						effectParam2 = 0xF0 | std::min(uint8(0x0F), effectParam2);	// Well, this is not too accurate...
+					case 1:  // Note Finetune (1/16th of a semitone signed 8-bit value, not 1/128th as the interface claims)
+						effect2 = (effectParam2 < 128) ? CMD_PORTAMENTOUP : CMD_PORTAMENTODOWN;
+						if(effectParam2 >= 128)
+							effectParam2 = ~effectParam2 + 1;
+						if(effectParam2 >= 16 && m->IsNote())
+						{
+							if(effect2 == CMD_PORTAMENTOUP)
+								m->note = static_cast<ModCommand::NOTE>(std::min(m->note + effectParam2 / 16, static_cast<int>(NOTE_MAX)));
+							else
+								m->note = static_cast<ModCommand::NOTE>(std::max(m->note - effectParam2 / 16, static_cast<int>(NOTE_MIN)));
+							effectParam2 %= 16u;
+						}
+						effectParam2 = 0xF0 | std::min(uint8(0x0F), effectParam2);
 						break;
 					case 2:		// Note Delay (wtf is the difference to Sample Delay?)
 						effectParam2 = DMFdelay2MPT(effectParam2, settings.internalTicks);
@@ -666,7 +673,7 @@ static PATTERNINDEX ConvertDMFPattern(FileReader &file, DMFPatternSettings &sett
 					case 4:		// Portamento Up
 					case 5:		// Portamento Down
 						effectParam2 = DMFporta2MPT(effectParam2, settings.internalTicks, true);
-						effect2 = static_cast<ModCommand::COMMAND>(effect2 == 4 ? CMD_PORTAMENTOUP : CMD_PORTAMENTODOWN);
+						effect2 = (effect2 == 4) ? CMD_PORTAMENTOUP : CMD_PORTAMENTODOWN;
 						useMem2 = true;
 						break;
 					case 6:		// Portamento to Note
