@@ -249,18 +249,19 @@ struct MO3Sample
 {
 	enum MO3SampleFlags
 	{
-		smp16Bit			= 0x01,
-		smpLoop				= 0x10,
-		smpPingPongLoop		= 0x20,
-		smpSustain			= 0x100,
-		smpSustainPingPong	= 0x200,
-		smpStereo			= 0x400,
-		smpCompressionMPEG	= 0x1000,					// MPEG 1.0 / 2.0 / 2.5 sample
-		smpCompressionOgg	= 0x1000 | 0x2000,			// Ogg sample
-		smpSharedOgg		= 0x1000 | 0x2000 | 0x4000,	// Ogg sample with shared vorbis header
-		smpDeltaCompression	= 0x2000,					// Deltas + compression
-		smpDeltaPrediction	= 0x4000,					// Delta prediction + compression
-		smpCompressionMask	= 0x1000 | 0x2000 | 0x4000
+		smp16Bit            = 0x01,
+		smpLoop             = 0x10,
+		smpPingPongLoop     = 0x20,
+		smpSustain          = 0x100,
+		smpSustainPingPong  = 0x200,
+		smpStereo           = 0x400,
+		smpCompressionMPEG  = 0x1000,                    // MPEG 1.0 / 2.0 / 2.5 sample
+		smpCompressionOgg   = 0x1000 | 0x2000,           // Ogg sample
+		smpSharedOgg        = 0x1000 | 0x2000 | 0x4000,  // Ogg sample with shared vorbis header
+		smpDeltaCompression = 0x2000,                    // Deltas + compression
+		smpDeltaPrediction  = 0x4000,                    // Delta prediction + compression
+		smpOPLInstrument    = 0x8000,                    // OPL patch data
+		smpCompressionMask  = 0x1000 | 0x2000 | 0x4000 | 0x8000
 	};
 
 	int32le  freqFinetune;	// Frequency in S3M and IT, finetune (0...255) in MOD, MTM, XM
@@ -374,39 +375,38 @@ struct MO3SampleChunk
 	} while(carry); \
 }
 
-static bool UnpackMO3Data(FileReader &file, uint8 *dst, uint32 size)
+
+static bool UnpackMO3Data(FileReader &file, std::vector<uint8> &uncompressed, const uint32 size)
 {
 	if(!size)
-	{
 		return false;
-	}
 
 	uint16 data = 0;
 	int8 carry = 0;			// x86 carry (used to propagate the most significant bit from one byte to another)
 	int32 strLen = 0;		// length of previous string
 	int32 strOffset;		// string offset
-	uint8 *initDst = dst;
-	uint32 ebp, previousPtr = 0;
-	uint32 initSize = size;
+	uint32 previousPtr = 0;
 
 	// Read first uncompressed byte
-	*dst++ = file.ReadUint8();
-	size--;
+	uncompressed.push_back(file.ReadUint8());
+	uint32 remain = size - 1;
 
-	while(size > 0)
+	while(remain > 0)
 	{
 		READ_CTRL_BIT;
 		if(!carry)
 		{
 			// a 0 ctrl bit means 'copy', not compressed byte
-			*dst++ = file.ReadUint8();
-			size--;
+			if(!file.CanRead(1))
+				break;
+			uncompressed.push_back(file.ReadUint8());
+			remain--;
 		} else
 		{
 			// a 1 ctrl bit means compressed bytes are following
-			ebp = 0; // length adjustment
-			DECODE_CTRL_BITS; // read length, and if strLen > 3 (coded using more than 1 bits pair) also part of the offset value
-			strLen -=3;
+			uint8 lengthAdjust = 0;  // length adjustment
+			DECODE_CTRL_BITS;        // read length, and if strLen > 3 (coded using more than 1 bits pair) also part of the offset value
+			strLen -= 3;
 			if(strLen < 0)
 			{
 				// means LZ ptr with same previous relative LZ ptr (saved one)
@@ -419,10 +419,10 @@ static bool UnpackMO3Data(FileReader &file, uint8 *dst, uint32 size)
 				strLen = 0;
 				strOffset = ~strOffset;
 				if(strOffset < -1280)
-					ebp++;
-				ebp++;	// length is always at least 1
+					lengthAdjust++;
+				lengthAdjust++;  // length is always at least 1
 				if(strOffset < -32000)
-					ebp++;
+					lengthAdjust++;
 				previousPtr = strOffset; // save current Ptr
 			}
 
@@ -437,36 +437,33 @@ static bool UnpackMO3Data(FileReader &file, uint8 *dst, uint32 size)
 				DECODE_CTRL_BITS;	// decode length: 1 is the most significant bit,
 				strLen += 2;		// then first bit of each bits pairs (noted n1), until n0.
 			}
-			strLen += ebp; // length adjustment
-			if(size >= static_cast<uint32>(strLen) && strLen > 0)
-			{
-				// Copy previous string
-				if(strOffset >= 0 || static_cast<std::ptrdiff_t>(dst - initDst) + strOffset < 0)
-				{
-					break;
-				}
-				size -= strLen;
-				const uint8 *string = dst + strOffset;
-				while(strLen > 0)
-				{
-					*dst++ = *string++;
-					strLen--;
-				}
-			} else
-			{
+			strLen += lengthAdjust;  // length adjustment
+
+			if(remain < static_cast<uint32>(strLen) || strLen <= 0)
 				break;
-			}
+			if(strOffset >= 0 || -static_cast<ptrdiff_t>(uncompressed.size()) > strOffset)
+				break;
+
+			// Copy previous string
+			// Need to do this in two steps as source and destination may overlap (e.g. strOffset = -1, strLen = 2 repeats last character twice)
+			uncompressed.insert(uncompressed.end(), strLen, 0);
+			remain -= strLen;
+			auto src = uncompressed.cend() - strLen + strOffset;
+			auto dst = uncompressed.end() - strLen;
+			do
+			{
+				strLen--;
+				*dst++ = *src++;
+			} while(strLen > 0);
 		}
 	}
 #ifdef MPT_BUILD_FUZZER
 	// When using a fuzzer, we should not care if the decompressed buffer has the correct size.
 	// This makes finding new interesting test cases much easier.
-	while(size-- > 0)
-	{
-		*dst++ = 0;
-	}
+	return true;
+#else
+	return remain == 0;
 #endif // MPT_BUILD_FUZZER
-	return (dst - initDst) == static_cast<std::ptrdiff_t>(initSize);
 }
 
 
@@ -717,7 +714,7 @@ static bool ValidateHeader(const MO3ContainerHeader &containerHeader)
 	{
 		return false;
 	}
-	if(containerHeader.musicSize <= sizeof(MO3FileHeader))
+	if(containerHeader.musicSize <= sizeof(MO3FileHeader) || containerHeader.musicSize >= uint32_max / 2u)
 	{
 		return false;
 	}
@@ -764,24 +761,25 @@ bool CSoundFile::ReadMO3(FileReader &file, ModLoadingFlags loadFlags)
 	}
 
 	const uint8 version = containerHeader.version;
-	const uint32 musicSize = containerHeader.musicSize;
 
-	uint32 compressedSize = uint32_max;
+	uint32 compressedSize = uint32_max, reserveSize = 1024 * 1024;  // Generous estimate based on biggest pre-v5 MO3s found in the wild (~350K music data)
 	if(version >= 5)
 	{
 		// Size of compressed music chunk
 		compressedSize = file.ReadUint32LE();
-#ifndef MPT_BUILD_FUZZER
 		if(!file.CanRead(compressedSize))
-		{
 			return false;
-		}
-#endif // !MPT_BUILD_FUZZER
+		// Generous estimate based on highest real-world compression ratio I found in a module (~20:1)
+		reserveSize = std::min(Util::MaxValueOfType(reserveSize) / 32u, compressedSize) * 32u;
 	}
 
-	std::vector<uint8> musicData(musicSize);
-
-	if(!UnpackMO3Data(file, musicData.data(), musicSize))
+	std::vector<uint8> musicData;
+	// We don't always reserve the whole uncompressed size as claimed by the module to guard against broken files
+	// that e.g. claim that the uncompressed size is 1GB while the MO3 file itself is only 100 bytes.
+	// As the LZ compression used in MO3 doesn't allow for establishing a clear upper bound for the maximum size,
+	// this is probably the only sensible way we can prevent DoS due to huge allocations.
+	musicData.reserve(std::min(reserveSize, containerHeader.musicSize.get()));
+	if(!UnpackMO3Data(file, musicData, containerHeader.musicSize))
 	{
 		return false;
 	}
@@ -1414,6 +1412,13 @@ bool CSoundFile::ReadMO3(FileReader &file, ModLoadingFlags loadFlags)
 				{
 					unsupportedSamples = true;
 				}
+			} else if(compression == MO3Sample::smpOPLInstrument)
+			{
+				OPLPatch patch;
+				if(sampleData.ReadArray(patch))
+				{
+					sample.SetAdlib(true, patch);
+				}
 			} else
 			{
 				unsupportedSamples = true;
@@ -1885,16 +1890,18 @@ bool CSoundFile::ReadMO3(FileReader &file, ModLoadingFlags loadFlags)
 	if(fileHeader.flags & MO3FileHeader::modplugMode)
 	{
 		// Apply some old ModPlug (mis-)behaviour
-		for(INSTRUMENTINDEX i = 1; i <= GetNumInstruments(); i++)
+		if(!m_dwLastSavedWithVersion)
 		{
-			if(ModInstrument *ins = Instruments[i])
+			// These fixes are only applied when the OpenMPT version number is not known, as otherwise the song upgrade feature will take care of it.
+			for(INSTRUMENTINDEX i = 1; i <= GetNumInstruments(); i++)
 			{
-				// Fix pitch / filter envelope being shortened by one tick
-				if(m_dwLastSavedWithVersion < MAKE_VERSION_NUMERIC(1, 20, 00, 00))
+				if(ModInstrument *ins = Instruments[i])
+				{
+					// Fix pitch / filter envelope being shortened by one tick (for files before v1.20)
 					ins->GetEnvelope(ENV_PITCH).Convert(MOD_TYPE_XM, GetType());
-				// Fix excessive pan swing range
-				if(m_dwLastSavedWithVersion < MAKE_VERSION_NUMERIC(1, 26, 00, 00))
+					// Fix excessive pan swing range (for files before v1.26)
 					ins->nPanSwing = (ins->nPanSwing + 3) / 4u;
+				}
 			}
 		}
 		if(m_dwLastSavedWithVersion < MAKE_VERSION_NUMERIC(1, 18, 00, 00))
