@@ -381,6 +381,8 @@ public:
 
 	typedef uint32 samplecount_t; // Number of rendered samples
 
+	static constexpr uint32 TICKS_ROW_FINISHED = uint32_max - 1u;
+
 public:	// for Editing
 #ifdef MODPLUG_TRACKER
 	CModDoc *m_pModDoc = nullptr; // Can be a null pointer for example when previewing samples from the treeview.
@@ -430,16 +432,17 @@ public:
 	ResamplingMode m_nResampling; // Resampling mode (if overriding the globally set resampling)
 	int32 m_nRepeatCount = 0;     // -1 means repeat infinitely.
 	ORDERINDEX m_nMaxOrderPosition;
-	ModChannelSettings ChnSettings[MAX_BASECHANNELS];	// Initial channels settings
+	ModChannelSettings ChnSettings[MAX_BASECHANNELS];  // Initial channels settings
 	CPatternContainer Patterns;
-	ModSequenceSet Order;								// Pattern sequences (order lists)
+	ModSequenceSet Order;  // Pattern sequences (order lists)
 protected:
-	ModSample Samples[MAX_SAMPLES];						// Sample Headers
+	ModSample Samples[MAX_SAMPLES];
 public:
-	ModInstrument *Instruments[MAX_INSTRUMENTS];		// Instrument Headers
-	MIDIMacroConfig m_MidiCfg;							// MIDI Macro config table
+	ModInstrument *Instruments[MAX_INSTRUMENTS];  // Instrument Headers
+	MIDIMacroConfig m_MidiCfg;                    // MIDI Macro config table
 #ifndef NO_PLUGINS
-	SNDMIXPLUGIN m_MixPlugins[MAX_MIXPLUGINS];			// Mix plugins
+	SNDMIXPLUGIN m_MixPlugins[MAX_MIXPLUGINS];  // Mix plugins
+	uint32 m_loadedPlugins = 0;                 // Not a PLUGINDEX because number of loaded plugins may exceed MAX_MIXPLUGINS during MIDI conversion
 #endif
 	char m_szNames[MAX_SAMPLES][MAX_SAMPLENAME];		// Sample names
 
@@ -959,6 +962,7 @@ protected:
 	void PortamentoFineMPT(ModChannel &chn, int);
 	void PortamentoExtraFineMPT(ModChannel &chn, int);
 	void NoteSlide(ModChannel &chn, uint32 param, bool slideUp, bool retrig) const;
+	std::pair<uint32, bool> GetVolCmdTonePorta(const ModCommand &m, uint32 startTick) const;
 	void TonePortamento(ModChannel &chn, uint32 param) const;
 	void Vibrato(ModChannel &chn, uint32 param) const;
 	void FineVibrato(ModChannel &chn, uint32 param) const;
