@@ -95,7 +95,10 @@ struct UpgradePatternData
 			{
 				if((m.param & 0x0F) != 0x00 && (m.param & 0x0F) != 0x0F && (m.param & 0xF0) != 0x00 && (m.param & 0xF0) != 0xF0)
 				{
-					m.param &= 0x0F;
+					if(m.command == CMD_GLOBALVOLSLIDE)
+						m.param &= 0xF0;
+					else
+						m.param &= 0x0F;
 				}
 			}
 
