@@ -276,7 +276,7 @@ void LFOPlugin::HardAllNotesOff()
 }
 
 
-bool LFOPlugin::IsNotePlaying(uint32 note, CHANNELINDEX trackerChn)
+bool LFOPlugin::IsNotePlaying(uint8 note, CHANNELINDEX trackerChn)
 {
 	if(IMixPlugin *plugin = GetOutputPlugin())
 		return plugin->IsNotePlaying(note, trackerChn);
@@ -292,7 +292,7 @@ void LFOPlugin::SaveAllParameters()
 		return;
 
 	m_pMixStruct->defaultProgram = -1;
-	m_pMixStruct->pluginData.assign(chunk.cbegin(), chunk.cend());
+	m_pMixStruct->pluginData.assign(chunk.begin(), chunk.end());
 }
 
 
@@ -362,6 +362,7 @@ void LFOPlugin::SetChunk(const ChunkData &chunk, bool)
 		m_bypassed = data.bypassed != 0;
 		m_outputToCC = data.outputToCC != 0;
 		m_oneshot = data.loopMode != 0;
+		RecalculateFrequency();
 	}
 }
 
