@@ -44,6 +44,7 @@ IMixPlugin::IMixPlugin(VSTPluginLib &factory, CSoundFile &sndFile, SNDMIXPLUGIN
 	, m_SndFile(sndFile)
 	, m_pMixStruct(mixStruct)
 {
+	m_SndFile.m_loadedPlugins++;
 	m_MixState.pMixBuffer = (mixsample_t *)((((intptr_t)m_MixBuffer) + 7) & ~7);
 	while(m_pMixStruct != &(m_SndFile.m_MixPlugins[m_nSlot]) && m_nSlot < MAX_MIXPLUGINS - 1)
 	{
@@ -71,6 +72,7 @@ IMixPlugin::~IMixPlugin()
 	if (m_pPrev) m_pPrev->m_pNext = m_pNext;
 	m_pPrev = nullptr;
 	m_pNext = nullptr;
+	m_SndFile.m_loadedPlugins--;
 }
 
 
@@ -930,8 +932,11 @@ void IMidiPlugin::MidiCommand(const ModInstrument &instr, uint16 note, uint16 vo
 }
 
 
-bool IMidiPlugin::IsNotePlaying(uint32 note, CHANNELINDEX trackerChn)
+bool IMidiPlugin::IsNotePlaying(uint8 note, CHANNELINDEX trackerChn)
 {
+	if(!ModCommand::IsNote(note) || trackerChn >= CountOf(m_MidiCh[GetMidiChannel(trackerChn)].noteOnMap[note]))
+		return false;
+
 	note -= NOTE_MIN;
 	return (m_MidiCh[GetMidiChannel(trackerChn)].noteOnMap[note][trackerChn] != 0);
 }
